@@ -2,7 +2,7 @@
 #define VERSION "0.20?"
 #endif
 static char c1[] = "     display-dhammapada V" VERSION " \n";
-static char c2[] = " (C) January 1997, 1998 by Ricardas Cepas <rch@pub.osf.lt>. \n";
+static char c2[] = " (C) January 1997-2001 by Ričardas Čepas (Richard Cepas) <rch@writeme.com> and contributors. \n";
 static char c3[] = " Copying policy: Public Domain.\n";
 static char c4[] = " No warranty. \n";
 static char c5[] = " ANSI C. Tested with GCC 2.7.2.3 \n";
@@ -15,25 +15,24 @@ static char c5[] = " ANSI C. Tested with GCC 2.7.2.3 \n";
 #define max_string_length 278
 #define mark_length 4
 #ifndef DPPATHNAME
-#define DPPATHNAME "/usr/local/lib/display-dhammapada/dhammapada-english-transl.txt"
+#define DPPATHNAME "/usr/local/lib/display-dhammapada/dhammapada-english-transl.txt:/usr/lib/display-dhammapada/dhammapada-english-transl.txt"
 #endif
-#ifndef DPPATHNAME2
-#define DPPATHNAME2 "/usr/lib/display-dhammapada/dhammapada-english-transl.txt"
+#ifndef DPPATHNAME_ALT
+#define DPPATHNAME_ALT "/usr/local/lib/display-dhammapada/dhammapada-alternate.txt:/usr/lib/display-dhammapada/dhammapada-alternate.txt"
 #endif
 
-static const char dp_pathname1[] = DPPATHNAME;
+static const char dp_pathname[] = DPPATHNAME ":dhammapada-english-transl.txt";
 static const char c6[] = "                                                                                  ";
-static const char dp_pathname2[] = DPPATHNAME2;
+static const char dp_pathname_alt[] = DPPATHNAME_ALT ":dhammapada-alternate.txt";
 static const char c7[] = "                                                                                  ";
-static const char dp_pathname3[] = "dhammapada-english-transl.txt";
-static const char c8[] = "                                                                                  ";
+const char bad_args[] = "Bad arguments";
 const char IO_err[] = "I/O error";
 const char cannot_open[] = "    -- cannot open any of the files.";
 const char too_long_verse[] = " file format error ?";
 const char unexp_EOF[] = "unexpected end of file.";
 
 
-const max_strings_in_verse = 20;
+const int max_strings_in_verse = 20;
 
 /*const long min_all_verses_size=50; */
 static struct
@@ -159,9 +158,13 @@ void
 Help ()
 {
      printf ("%s Displays a random verse \n", c1);
-     printf ("from %s \nor from %s \nor from %s file. \n", dp_pathname1, dp_pathname2, dp_pathname3);
-     printf (" Arguments: \n           <number>         specify number of verse(s) to display \n           all              display all file at once \n");
-     printf (" Arguments for debugging only: \n           0x<hex_number>   specify internal index of verse(s) to display \n           0xall            display all file at once \n");
+     printf ("from John Richards's translation:\n%s \nor from F. Max Muller's translation:\n%s\n", dp_pathname, dp_pathname_alt);
+     printf (" Arguments: \n"
+	     "           -r               Use John Richard's translation (default)\n"
+	     "           -m               Use F. Max Muller's translation\n"
+	     "           -b               Display the same verse(s) from both\n"
+	     "           <number>         specify the number of the verse(s) to display \n"
+	     "           all              display all file at once \n");
      printf ("%s%s%s", c2, c3, c4);
 }
 
@@ -219,52 +222,104 @@ Random_index ()
      return (random_index);
 }
 
+void Filename_num (char* filename, const char* filename_list, int num)
+{
+	int i;
+	for (i = 1; i < num; i++) {	
+		while (*filename_list != ':' && *filename_list != '\0') 
+			filename_list++;
+		if (*filename_list == '\0') {
+			*filename = '\0';
+			return;
+		}
+		filename_list++;
+	}
+	while (*filename_list != ':' && *filename_list != '\0')
+		*filename++ = *filename_list++;
+	*filename = '\0';
+}
+
 int
-main (int argc, char *argv[], char *env[])
+main (int argc, char *argv[])
 {
      FILE *dp;
-     int index;
+     int index = -1;
      char *stop_char_ptr;
+     char dp_filename [200];
+     enum {MULLER, RICHARDS, BOTH} dhammapada_version = RICHARDS;
+     int filename_number;
+     int is_all = 0;
 
-     if ((dp = fopen (dp_pathname1, "rt")) == NULL)
-        if ((dp = fopen (dp_pathname2, "rt")) == NULL)
-          if ((dp = fopen (dp_pathname3, "rt")) == NULL)
-            {
-                 fprintf (stderr, " %s \n %s \n %s \n", dp_pathname1, dp_pathname2, dp_pathname3);
-                 Error (cannot_open);
-            }
-
-     if (argc > 1)
-       {
-            if (strcmp (argv[1], "0xall") == 0)
-                 for (index = 0; index < sizeof marks / sizeof marks[0]; index++)
-                   {
-                        printf ("0x%X \n", index);
-                        Get_and_print_verse (dp, index);
-                   }
-	    else if (strcmp (argv[1], "all") == 0)
-                 for (index = 0; index < sizeof marks / sizeof marks[0]; index++)
-		      Get_and_print_verse (dp, index);
-            else
-	    {
-	      errno = 0;
-	      index = strtol (argv[1], &stop_char_ptr, 0);
-	      if (errno != ERANGE && *stop_char_ptr == '\0' && argv[1][0] != '\0')
-		if (argv[1][0] == '0' && (argv[1][1] == 'x' || argv[1][1] == 'X'))
-		{
-		    printf ("0x%X \n", index);
-		    Get_and_print_verse (dp, index);
+     /* Split the options and set the booleans */
+     if (argc > 1) {
+	     if (strcmp (argv[1], "-b") == 0) dhammapada_version = BOTH;
+	     else if (strcmp (argv[1], "-r") == 0) dhammapada_version = RICHARDS;
+	     else if (strcmp (argv[1], "-m") == 0) dhammapada_version = MULLER;
+	     else if (strcmp (argv[1], "-h") == 0 || strcmp (argv[1], "-?") == 0) {
+	        Help ();
+		exit (0);
+	     }
+	     else if (!(is_all = ((strcmp (argv[1], "all") == 0)))) {
+		errno = 0;
+		index = strtol (argv[1], &stop_char_ptr, 0);
+		if (errno == ERANGE || *stop_char_ptr != '\0') { 
+			index = -1;
+			Help ();
+			Error (bad_args);
 		}
-		else
-		{
-		    Get_and_print_verse_number_n (dp, index);
+	     }
+	     if (argc > 2) {
+		if (index == -1 || ! is_all) {
+	     		if (!(is_all = ((strcmp (argv[2], "all") == 0)))) {
+				errno = 0;
+				index = strtol (argv[2], &stop_char_ptr, 0);
+				if (errno == ERANGE || *stop_char_ptr != '\0') 
+					index = -1;
+			}
 		}
-            else
-                 Help ();
-	    }
-       }
-     else
-          Get_and_print_verse (dp, Random_index ());
+		else {
+			Help ();
+			Error (bad_args);
+		}
+	     }
+	}
+open_file:
+     /* Open the file (Richards if both) */
+     dp = NULL;
+     dp_filename[0] = ' ';
+     filename_number = 1;
+     while (dp == NULL) {
+	     
+	     if (dhammapada_version == MULLER) {
+	     	Filename_num (dp_filename, dp_pathname_alt, filename_number);
+	     }
+	     else Filename_num (dp_filename, dp_pathname, filename_number);
+	     
+	     if (dp_filename[0] == '\0') {
+		     if (dhammapada_version == MULLER) 
+			     fprintf (stderr, "%s\n", dp_pathname_alt);
+		     else
+			fprintf (stderr, "%s\n", dp_pathname);	     
+		     Error (cannot_open);
+	     }
+	     
+	     dp = fopen (dp_filename, "rt");
+	     filename_number++;
+     }
+     /* Print the verse(s) */
+     if (is_all) 
+     	for (index = 0; index < sizeof marks / sizeof marks[0]; index++)
+		Get_and_print_verse (dp, index);
+     else if (index != -1)
+	Get_and_print_verse_number_n (dp, index);
+     else 
+        Get_and_print_verse (dp, Random_index ());
+     /* Close the file */
      fclose (dp);
-     exit (0);
+     /* If both, set dhammapada_version to MULLER and goto open_file */
+     if (dhammapada_version == BOTH) {
+	     dhammapada_version = MULLER;
+	     goto open_file;
+     }
+     exit (0); 
 }
